
# q-expansion of newform 7245.2.a.j, downloaded from the LMFDB on 08 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 7245
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [1], [1], [-3], [-4], [6], [-1], [1], [-6], [8], [-10], [3], [-10], [3], [9], [3], [11], [-10], [0], [2], [-10], [0], [0], [2], [3], [-1], [12], [-10], [6], [-13], [-15], [-3], [-16], [-3], [-19], [17], [11], [15], [-6], [-12], [-10], [15], [-25], [-12], [-13], [5], [-16], [-24], [-13], [-6], [30], [17], [24], [-3], [-21], [-18], [26], [-19], [-18], [32], [12], [32], [-33], [-19], [-30], [-7], [8], [6], [8], [18], [0], [-19], [26], [-34], [-36], [-18], [8], [-15], [20], [24], [14], [-27], [-25], [8], [-18], [6], [-28], [-18], [-25], [-30], [-12], [8], [0], [-16], [-18], [27], [-24], [-31], [-13], [-28], [6], [0], [-15], [-16], [-4], [27], [-18], [-36], [-10], [38], [8], [18], [-28], [-16], [-3], [-1], [12], [-36], [-36], [-13], [23], [12], [-24], [-28], [51], [8], [24], [47], [-10], [32], [-33], [-4], [8], [45], [-34], [-36], [-25], [24], [36], [-16], [18], [-7], [3], [38], [24], [44], [-3], [14], [-24], [5], [48], [-28], [-36], [-28], [12], [-4], [6], [23], [24], [42], [21], [8], [12], [21], [42], [41], [2], [44], [-15], [-3], [-22], [57], [2], [-22], [54], [-37], [-6], [41], [-10], [53], [42], [-25], [9], [39], [42], [-13], [-40], [41], [24], [38], [48], [2], [-57], [-33], [66], [56], [14], [-42], [3], [-9], [-10], [-52], [-13], [-18], [51], [20], [-54], [-12], [50], [-49], [-15], [20], [-18], [0], [-10], [-7], [6], [-60], [-36], [23], [-49], [36], [-52], [-33], [41], [-66], [30], [8], [48], [-13], [44], [50], [60], [-37], [-54], [-7], [-54], [-18], [48], [21], [2], [50], [-1], [69], [-57], [-16], [12], [8], [9], [50], [-12], [33], [-73], [-78], [66], [-46], [-25], [-21], [-46], [-28], [-57], [38], [29], [15], [-1], [39], [-60], [-46], [-6], [38], [38], [26], [20], [38], [8], [-66], [59], [-52], [42], [-21], [-52], [-6], [2], [62], [-6], [-34], [-27], [14], [42], [-27], [-12], [36], [0], [14], [-72], [8], [15], [75], [-46], [-22], [42], [56], [-66], [86], [-31], [-72], [5], [9], [-82], [-39], [-54], [15], [-16], [75], [-43], [-9], [30], [-1], [30], [17], [20], [27], [29], [45], [-43], [38], [-28], [-45], [87], [-82], [0], [-16], [-21], [-58], [-87], [74], [90], [-40], [32], [-10], [-57], [45], [-64], [45], [-78], [20], [-7], [78], [-30], [-40], [17], [24], [-37], [-10], [69], [-18], [6], [-78], [-72], [80], [21], [-6], [-33], [20], [-91], [15], [56], [-22], [-78], [8], [-6], [-63], [-7], [-34], [-24], [-21], [11], [30], [44], [0], [-6], [-43], [78], [44], [69], [-73], [-40], [50], [-90], [26], [33], [24], [-22], [39], [86], [-25], [24], [62], [96], [62], [0], [20], [-78], [-15], [20], [2], [-6], [-76], [12], [20], [60], [-45], [-10], [-52], [-30], [-72], [-106], [84], [24], [-66], [83], [30], [-48], [14], [-3], [105], [78], [20], [-45]]
